unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    Button1: TButton;
    OpenDialog1: TOpenDialog;
    Image1: TImage;
    Image2: TImage;
    Image3: TImage;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Button2: TButton;
    Label6: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private 錾 }
  public
    { Public 錾 }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
    Info;

type
    TDelphiResource = class(TObject)
    protected
        FHandle: THandle;
        FPath: String;
        FResList: TStringList;
        FErrList: TStringList;
        FSavePath: String;
        function OpenResource(): Boolean;
        procedure CloseResource();
        function GetResourceNames(): Integer;
        procedure RestoreUnit(const ADfmList: TStrings; const AFormName: String);
        function GetResourceFormFile(AList: TStrings; const AFormName: String) : Boolean;
    public
        constructor Create();
        destructor Destroy(); override;
        function Scan(const APath: String): Boolean;
        property ErrList: TStringList read FErrList;
        property SavePath: String read FSavePath;
    end;

//-----------------------------------------------------------------------------
//  \[X񋓗pR[obN֐
function EnumFunc(Instance: THandle; ResType: Integer; ResName: PChar; lParam: LongInt): Boolean; stdcall;
var
    LFound: HRSRC;
    LLoaded: HGLOBAL;
    LBuffer: PChar;
begin
    //  \[X
    LFound := FindResource(Instance, ResName, RT_RCDATA);
    //  ǂݍ
    LLoaded := LoadResource(Instance, LFound);
    //  bN
    LBuffer := LockResource(LLoaded);
    //  tH[\[X̂݋L^
    if (Pos('TPF0', LBuffer) = 1) then
    begin
        TStringList(lParam).Add(ResName);
    end;
    //  񋓑s
    Result := True;
end;

//-----------------------------------------------------------------------------
//  \[X
function TDelphiResource.GetResourceNames(): Integer;
begin
    FResList.Clear();
    if (FHandle <> 0) then
    begin
        try
            EnumResourceNames(FHandle, RT_RCDATA, @EnumFunc, LPARAM(FResList));
        except
            on E: Exception do
            begin
                FErrList.Add(BuildErrorMessage(E, ' in GetResourceNames()'));
            end; 
        end;
    end;
    Result := FResList.Count;
end;

//-----------------------------------------------------------------------------
//  
function TDelphiResource.OpenResource(): Boolean;
begin
    FHandle := LoadLibraryEx(PChar(FPath), 0, LOAD_LIBRARY_AS_DATAFILE);
    Result := (FHandle <> 0);
    if (not Result) then
    begin
        FErrList.Add(BuildErrorMessage(nil, ' in OpenResource()'));
    end;
end;

//-----------------------------------------------------------------------------
//  ߂
procedure TDelphiResource.CloseResource();
begin
    if (FHandle <> 0) then
    begin
        if (not FreeLibrary(FHandle)) then
        begin
            FErrList.Add(BuildErrorMessage(nil, ' in CloseResource()'));
        end;
        FHandle := 0;
    end;
end;

//-----------------------------------------------------------------------------
//  RXgN^
constructor TDelphiResource.Create();
begin
    inherited;
    FResList := TStringList.Create();
    FErrList := TStringList.Create();
end;

//-----------------------------------------------------------------------------
//  fXgN^
destructor TDelphiResource.Destroy();
begin
    CloseResource();
    FResList.Free();
    FErrList.Free();
    inherited;
end;

//-----------------------------------------------------------------------------
//  tH[\[X烆jbgt@CXgA
procedure TDelphiResource.RestoreUnit(const ADfmList: TStrings; const AFormName: String);
var
    LDfmName: String;
    LUnitName: String;
    LList: TStringList;
    i: Integer;
    LLine: String;
begin
    FSavePath := ExtractFilePath(ParamStr(0)) + StringReplace(ExtractFileName(FPath), '.', '_', [rfReplaceAll]) + '\';
    ForceDirectories(FSavePath);

    LDfmName := 'Unit' + AFormName + '.dfm';
    LUnitName := 'Unit' + AFormName + '.pas';

    LList := TStringList.Create();

    //  tH[t@Cۑ
    ADfmList.SaveToFile(FSavePath + LDfmName);

    //  jbgt@CCČ
    LList.Add(Format('unit %s;', ['Unit' + AFormName]));
    LList.Add('');
    LList.Add('interface');
    LList.Add('');
    LList.Add('uses');
    LList.Add('  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,');
    LList.Add('  Dialogs;');
    LList.Add('');
    LList.Add('type');
    LList.Add(Format('  %s = class(TForm)', [AFormName]));

    //  擪ŝāAobject Ŏn܂sR|[lg錾
    for i := 1 to ADfmList.Count - 1 do
    begin
        LLine := Trim(ADfmList.Strings[i]);
        if (Pos('object', LLine) = 1) then
        begin
            Delete(LLine, 1, Length('object'));
            LList.Add('    ' + Trim(LLine) + ';');
        end;
    end;

    LList.Add('  private');
    LList.Add('    { Private 錾 }');
    LList.Add('  public');
    LList.Add('    { Public 錾 }');
    LList.Add('  end;');
    LList.Add('');
    LList.Add('var');
    LList.Add(Format('  %s: %s;', [StringReplace(AFormName, 'T', '', [rfIgnoreCase]), AFormName]));
    LList.Add('');
    LList.Add('implementation');
    LList.Add('');
    LList.Add('{$R *.dfm}');
    LList.Add('');
    LList.Add('end.');

    //  jbgt@Cۑ
    LList.SaveToFile(FSavePath + LUnitName);

    LList.Free();
end;

//-----------------------------------------------------------------------------
//  tH[\[Xo
function TDelphiResource.GetResourceFormFile(AList: TStrings; const AFormName: String) : Boolean;
var
    LIn: TMemoryStream;
    LOut: TMemoryStream;
    LSize: Integer;
    LHeader: String;
    LBuffer: PChar;
    LEndIndex: Integer;
    i: Integer;
var
    LFound: HRSRC;
    LLoaded: HGLOBAL;
begin
    Result := True;
    LIn := TMemoryStream.Create();
    LOut := TMemoryStream.Create();

    //  \[X
    LFound := FindResource(FHandle, PChar(AFormName), RT_RCDATA);
    //  ǂݍ
    LLoaded := LoadResource(FHandle, LFound);
    //  bN
    LBuffer := LockResource(LLoaded);

    if (Pos('TPF0', LBuffer) <> 0) then
    begin
        //  TCY
        LSize := SizeOfResource(FHandle, LFound);
        //  wb_
        LHeader := #$FF + Char(RT_RCDATA) + #$00 + AFormName +
                   #$00#$30#$10 +
                   Char(LoByte(LoWord(LSize))) +
                   Char(HiByte(LoWord(LSize))) +
                   Char(LoByte(HiWord(LSize))) +
                   Char(HiByte(HiWord(LSize)));

        //  Xg[TCYZbg
        LIn.Size := LSize;
        LOut.Size := 2 * LSize;     //  TZ

        //  wb_
        LIn.Write(LHeader[1], Length(LHeader));
        //  f[^
        LIn.Write(LBuffer^, LSize);
        //  ߂
        LIn.Seek(0, soFromBeginning);

        //  \[Xf[^eLXgϊ(邽߂Ƀwb_s)
        try
            ObjectResourceToText(LIn, LOut);
        except
            on E:Exception do
            begin
                FErrList.Add(BuildErrorMessage(E, ' in GetResourceFormFile(' + AFormName + ')'));
                Result := False;
            end;
        end;

        //  ߂
        LOut.Seek(0, soFromBeginning);

        //  ]L
        AList.BeginUpdate();
        AList.LoadFromStream(LOut);

        //  ݐ
        LEndIndex := AList.IndexOf('end');
        for i := AList.Count - 1 downto LEndIndex + 1 do
        begin
            AList.Delete(i);
        end;

        //  jbgč\
        RestoreUnit(AList, AFormName);
        
        AList.EndUpdate();
    end;

    LIn.Free();
    LOut.Free();
end;

//-----------------------------------------------------------------------------
//  `FbNs
function TDelphiResource.Scan(const APath: String): Boolean;
var
    i, LCount: Integer;
    LList: TStringList;
begin
    Result := True;
    FPath := APath;
    LList := TStringList.Create();

    //  JS}
    if (OpenResource()) then
    begin
        //  \[X
        LCount := GetResourceNames();
        for i := 0 to LCount - 1 do
        begin
            //  tH[\[Xo
            if (not GetResourceFormFile(LList, FResList.Strings[i])) then
            begin
                Result := False;
            end;
        end;

        CloseResource();
    end;

    LList.Free();
end;

//-----------------------------------------------------------------------------
//  N
procedure TForm1.FormCreate(Sender: TObject);
begin
    SetFormCaption(Self);
end;

//-----------------------------------------------------------------------------
//  `FbNs
procedure TForm1.Button1Click(Sender: TObject);
begin
    if (OpenDialog1.Execute()) then
    begin
        with TDelphiResource.Create() do
        begin
            //  XLs
            if (not Scan(OpenDialog1.FileName)) then
            begin
                //  s烁bZ[W\
                AppMessageBox(ErrList.Text, MB_OK or MB_ICONWARNING);
            end;

            //  ȂɂۑĂ悤Ȃ
            if (DirectoryExists(SavePath)) then
            begin
                if (AppMessageBox('ot@CۑtH_J܂H'#13#10 + SavePath,
                            MB_YESNO or MB_ICONQUESTION) = ID_YES) then
                begin
                    SimpleShellExecute(SavePath);
                end;
            end
            //  Ȃ
            else
            begin
                AppMessageBox('΂DelphȉꍇA'#13#10
                            + 'EUPX ȂǂňkĂH'#13#10
                            + 'EDelphi6 o[WăeXgłĂȂH'#13#10
                            + '̂ǂ炩Ǝv܂B݂܂B', MB_OK or MB_ICONINFORMATION);
            end;

            Free();
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  o[W\
procedure TForm1.Button2Click(Sender: TObject);
begin
    OpenInfoForm();
end;

end.
